# ParaView

Open-Source, Multi-Platform Data Analysis and Visualization Application

## Introduction

**ParaView** is an open-source, multi-platform data analysis and visualization application. ParaView users can quickly build visualizations to analyze their data using qualitative and quantitative techniques. The data exploration can be done interactively in 3D or programmatically using ParaView's batch processing capabilities.

ParaView was developed to analyze extremely large datasets using distributed memory computing resources. It can be run on supercomputers to analyze datasets of exascale size as well as on laptops for smaller data.

Homepage : <http://www.paraview.org/>

## Installed Version

Currently, version 5.1.2 compiled with intel/2017a against intel MPI library and OSMesa 12.0.2 is installed on Anselm.

## Usage

On Anselm, ParaView is to be used in client-server mode. A parallel ParaView server is launched on compute nodes by the user, and client is launched on your desktop PC to control and view the visualization. Download ParaView client application for your OS here: <http://paraview.org/paraview/resources/software.php>. 

!!!Warning
    Your version must match the version number installed on Anselm.

### Launching Server

To launch the server, you must first allocate compute nodes, for example

```console
$ qsub -I -q qprod -A OPEN-0-0 -l select=2
```

to launch an interactive session on 2 nodes. Refer to [Resource Allocation and Job Execution](../job-submission-and-execution/) for details.

After the interactive session is opened, load the ParaView module :

```console
$ ml ParaView/5.1.2-intel-2017a-mpi
```

Now launch the parallel server, with number of nodes times 16 processes:

```console
$ mpirun -np 32 pvserver --use-offscreen-rendering
    Waiting for client...
    Connection URL: cs://cn77:11111
    Accepting connection(s): cn77:11111
```

Note the that the server is listening on compute node cn77 in this case, we shall use this information later.

### Client Connection

Because a direct connection is not allowed to compute nodes on Anselm, you must establish a SSH tunnel to connect to the server. Choose a port number on your PC to be forwarded to ParaView server, for example 12345. If your PC is running Linux, use this command to establish a SSH tunnel:

```console
$ ssh -TN -L 12345:cn77:11111 username@anselm.it4i.cz
```

replace username with your login and cn77 with the name of compute node your ParaView server is running on (see previous step). 

If you use PuTTY on Windows, load Anselm connection configuration, then go to *Connection* -> *SSH* -> *Tunnels* to set up the port forwarding.

Fill the Source port and Destination fields. **Do not forget to click the Add button.**

![](../../img/paraview_ssh_tunnel.png "SSH Tunnel in PuTTY")

Now launch ParaView client installed on your desktop PC. Select *File* -> *Connect*... and fill in the following :

![](../../img/paraview_connect.png "ParaView - Connect to server")

The configuration is now saved for later use. Now click Connect to connect to the ParaView server. In your terminal where you have interactive session with ParaView server launched, you should see:

```console
Client connected.
```

You can now use Parallel ParaView.

### Close Server

Remember to close the interactive session after you finish working with ParaView server, as it will remain launched even after your client is disconnected and will continue to consume resources.

## GPU Support

Currently, GPU acceleration is not supported in the server and ParaView will not take advantage of accelerated nodes on Anselm. Support for GPU acceleration might be added in the future.
